package com.github.huangp.learning.ga.travellingsalemanproblem;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Patrick Huang
 *         <a href="mailto:pahuang@redhat.com">pahuang@redhat.com</a>
 */
public class PopulationOfTours implements Population<CandidateTour> {
    private List<CandidateTour> tours;

    private PopulationOfTours(int populationSize) {
        tours = new ArrayList<CandidateTour>(populationSize);
    }

    public static PopulationOfTours firstGeneration(int populationSize) {
        PopulationOfTours population =
                new PopulationOfTours(populationSize);
        for (int i = 0; i < populationSize; i++) {
            population.tours.add(CandidateTour.newTour());
        }
        return population;
    }

    public static PopulationOfTours emptyPopulation(int populationSize) {
        PopulationOfTours pop =
                new PopulationOfTours(populationSize);
        for (int i = 0; i < populationSize; i++) {
            pop.tours.add(null);
        }
        return pop;
    }

    @Override
    public int size() {
        return tours.size();
    }

    @Override
    public CandidateTour getIndividual(int index) {
        return tours.get(index);
    }

    @Override
    public void setIndividual(int index, CandidateTour individual) {
        tours.set(index, individual);
    }

    @Override
    public CandidateTour getFittest() {
        CandidateTour fittest = tours.get(0);
        for (CandidateTour candidateTour : tours) {
            if (Double.compare(candidateTour.fitness(), fittest.fitness()) > 0) {
                fittest = candidateTour;
            }
        }
        return fittest;
    }
}
