package com.github.huangp.learning.datastructure;

import java.util.Arrays;
import java.util.List;

import org.assertj.core.api.Assertions;
import org.junit.Test;

/**
 * @author Patrick Huang
 *         <a href="mailto:pahuang@redhat.com">pahuang@redhat.com</a>
 */
public class GraphTest {
    /*
    * 0 --- 1
    * |   / | \
    * | /   |  2
    * |/    | /
    * 4 --- 3
    */
    @Test
    public void testBreadthFirstTraverse() {
        Graph<Integer> graph = new Graph<Integer>(false);
        graph.insertEdge(0, 1, false);
        graph.insertEdge(0, 4, false);
        graph.insertEdge(1, 4, false);
        graph.insertEdge(1, 3, false);
        graph.insertEdge(1, 2, false);
        graph.insertEdge(2, 3, false);
        graph.insertEdge(3, 4, false);

//        graph.print();
        Assertions.assertThat(graph.getNumOfEdges()).isEqualTo(7);

        graph.BFS(1);
    }

    /*
    * 0 ------ 1 --- 2
    * |\     /       |
    * | \   /        |
    * |  \ /         |
    * |   4 -------- 3
    * 5
    */
    @Test
    public void testFindPath() {
        Graph<Integer> graph = new Graph<Integer>(false);
        graph.insertEdge(0, 1, false);
        graph.insertEdge(0, 4, false);
        graph.insertEdge(0, 5, false);
        graph.insertEdge(1, 2, false);
        graph.insertEdge(1, 4, false);
        graph.insertEdge(2, 3, false);
        graph.insertEdge(3, 4, false);

        Assertions.assertThat(graph.getNumOfEdges()).isEqualTo(7);

        graph.BFS(0);

        System.out.println(" parent:" + graph.getParent());
        List<Integer> path = graph.findPath(0, 3);

        System.out.println("path: " + path);
    }

    /*
    * 0 ------ 1 --- 2
    * |\     /       |
    * | \   /        |
    * |  \ /         |
    * |   4 -------- 3
    * 5
    */
    @Test
    public void testDepthFirstSearch() {
        Graph<Integer> graph = new Graph<Integer>(false);
        graph.insertEdge(0, 1, false);
        graph.insertEdge(0, 4, false);
        graph.insertEdge(0, 5, false);
        graph.insertEdge(1, 2, false);
        graph.insertEdge(1, 4, false);
        graph.insertEdge(2, 3, false);
        graph.insertEdge(3, 4, false);

        Assertions.assertThat(graph.getNumOfEdges()).isEqualTo(7);

        graph.DFS(0);
        System.out.println("entry time:" + graph.getEntry());
        System.out.println(" exit time:" + graph.getExit());
    }
}
