package com.github.huangp.learning.datastructure;

import java.util.List;

import org.assertj.core.api.Assertions;
import org.junit.Test;

public class WeightedGraphTest {
    /*
    * 0 --- 1
    * |   / | \
    * | /   |  2
    * |/    | /
    * 4 --- 3
    */
    @Test
    public void testBreadthFirstTraverse() {
        WeightedGraph<Integer> graph = new WeightedGraph<Integer>(false);
        graph.insertEdge(0, 1, 1);
        graph.insertEdge(0, 4, 1);
        graph.insertEdge(1, 4, 2);
        graph.insertEdge(1, 3, 3);
        graph.insertEdge(1, 2, 4);
        graph.insertEdge(2, 3, 5);
        graph.insertEdge(3, 4, 3);

//        graph.print();
        Assertions.assertThat(graph.getNumOfEdges()).isEqualTo(7);

        graph.BFS(1);
    }

    /*
    * 0 ------ 1 --- 2
    * |\     /       |
    * | \   /        |
    * |  \ /         |
    * |   4 -------- 3
    * 5
    */
    @Test
    public void testFindPath() {
        WeightedGraph<Integer> graph = new WeightedGraph<Integer>(false);
        graph.insertEdge(0, 1, 1);
        graph.insertEdge(0, 4, 2);
        graph.insertEdge(0, 5, 3);
        graph.insertEdge(1, 2, 4);
        graph.insertEdge(1, 4, 2);
        graph.insertEdge(2, 3, 3);
        graph.insertEdge(3, 4, 1);

        Assertions.assertThat(graph.getNumOfEdges()).isEqualTo(7);

        graph.BFS(0);

        System.out.println(" parent:" + graph.getParent());
        List<Integer> path = graph.findPath(2, 3);

        System.out.println("path: " + path);
    }


}
